<?php
/**
 * Implementation of hook_form_alter()
 *
 * Adds the list of Related Products to a node_product edit form.
 */
function uc_upsell_form_alter(&$form, $form_state, $form_id) {
  global $user;

  // Add form associations field to base - but only for users with given permissions.
  if (user_access('administer upsell settings')) {

    // reduce to form's node type
    $node_type = preg_replace('/_node_form$/i', '', $form_id);
    // get list of product types
    $product_types = array_keys(uc_product_node_info());
    $product_types[] = 'product_kit';

    // Get all currently associated products - besides this one...
    if (in_array($node_type, $product_types)) {
      $current_prods_array = db_result(db_query("SELECT related_products FROM {uc_upsell_products} WHERE nid = %d", intval(arg(1))));
      $current_prods_array = unserialize($current_prods_array);
      $current_nids = array();

      if (is_array($current_prods_array)) {
        foreach ($current_prods_array AS $key => $current_product) {
          $current_nids[] = $current_product;
        }
      }

      // Alphabetize the array (since there's no telling by nid)
      (is_array($current_products)) ? asort($current_products) : '' ;

      // Get product types
      $config = uc_upsell_get_config();
      $product_types = array_values($config['global']['types']);
      $products_query = "('". implode("', '", $product_types) ."')";

      // Get all products
      $avail_products = array();
      $available_models = db_query("SELECT n.title, n.nid
        FROM {node} AS n
        WHERE n.type IN $products_query
        AND n.status = 1 AND n.nid != %d
        ORDER BY n.title", intval(arg(1)));

      if ($available_models) {
        while ($file_products = db_fetch_object($available_models)) {
          $avail_products[$file_products->nid] = $file_products->title;
        }
      }

      $assoc_form = array(
        '#type' => 'select',
        '#multiple' => TRUE,
        '#title' => t('Related Products for Upsell'),
        '#weight' => 30,
        '#description' => t('Select related products to offer at checkout.'),
        '#options' => $avail_products,
        '#default_value' => $current_nids,
        '#id' => 'upsell',
        );

      $form['base']['relatedProducts'] = $assoc_form;
    }
  }
}


/**
 * Submit the node form
 *
 * (When a product is added to the list of "related products" for a node, the system
 *  will go to the newly-associated "related" product, and make a reciprocal association.)
 *
 *  Ex: When editing Product A, add Product B as related.
 *  Upon form submission, uc_upsell also adds Product A to Product B's list of related products.
 */
function uc_upsell_form_submit(&$node) {
  global $user;

  // Get list of product types
  $product_types = array_keys(uc_product_node_info());
  // Include product kits also
  $product_types[] = 'product_kit';

  // Create an array of selected related products and add the current product to the related products lists for each of them
  if (in_array($node->type, $product_types) && is_array($node->relatedProducts)) {

    foreach ($node->relatedProducts as $product) {
      $ary = array();
      $ary[] = $node->nid;
      $products[] = $product;

      $related = db_result(db_query("SELECT related_products FROM {uc_upsell_products} WHERE nid = %d", $product));

      if (!$related) {
        db_query("INSERT INTO {uc_upsell_products} (nid, related_products) VALUES (%d, '%s')", $product, serialize($node->nid));
      }
      else {
        if (is_array($ary)) {
          if (is_array(unserialize($related))) {
            $related = unserialize($related);
            $ary = array_merge($ary, $related);
            $ary = array_unique($ary);
            $ary = serialize($ary);
          }
        }
        db_query("UPDATE {uc_upsell_products} SET related_products = '%s' WHERE nid = %d", $ary, $product);
      }
    }

    // Loop through the array of associate nodes and update them with the whole array.
    if (is_array($products) && count($products > 0)) {
      foreach ($products AS $key => $value) {
        $currentnids = db_result(db_query("SELECT related_products FROM {uc_upsell_products} WHERE nid = %d", $value));
        if (is_array(unserialize($currentnids))) {
          $merge = array_merge(unserialize($currentnids), $products);
          //drupal_set_message('current: '.print_r($currentnids ,true));
          //drupal_set_message('should update '.print_r($value, true).' with '.print_r($merge,true));
          db_query("UPDATE {uc_upsell_products} SET related_products = '%s' WHERE nid = %d", serialize($merge), $value);
        }
      }
    }

    // Get the list of related products for the current product
    $currentnids = db_result(db_query("SELECT related_products FROM {uc_upsell_products} WHERE nid = %d", $node->nid));

    if (is_array(unserialize($currentnids)) && count(unserialize($currentnids)) > 0) {
      $currentnids = unserialize($currentnids);
      // Delete current product from the related products list for any products that are being removed
      foreach ($currentnids as $currentnid) {
        if (!in_array($currentnid, $products)) {
          $nidarray = array();
          $nidarray[] = $node->nid;
          $updatenids = db_result(db_query("SELECT related_products FROM {uc_upsell_products} WHERE nid = %d", $currentnid));
          if (is_array(unserialize($updatenids)) && count(unserialize($updatenids)) > 0) {
            $updatenids = array_unique(unserialize($updatenids));
            $updatenids = array_diff($updatenids, $nidarray);
          }
          db_query("UPDATE {uc_upsell_products} SET related_products = '%s' WHERE nid = %d", serialize($updatenids), $currentnid);
        }
      }
    }

    if (is_array($products) && count($products) > 0) {
      // Decide whether we are updating the related products entry or inserting a new one.
      if (!db_result(db_query("SELECT related_products FROM {uc_upsell_products} WHERE nid = %d", $node->nid))) {
        db_query("INSERT INTO {uc_upsell_products} (nid, related_products) VALUES (%d, '%s')", $node->nid, serialize($products));
      }
      else {
        db_query("UPDATE {uc_upsell_products} SET `related_products` = '%s' WHERE `nid` = %d", serialize($products), $node->nid);
      }
      drupal_set_message(t('Related products were updated.'));
    }

  }
}


/**
 * Administer the related products area
 */
function upsell_products_admin() {
  $config = uc_upsell_get_config();
  $header = array(
    array('data' => t('Product')),
    array('data' => t('Related items')),
    'data' => t('Action'),
  );

  $query = "SELECT n.title, p.nid, up.related_products FROM {uc_products} as p
      LEFT JOIN {node} AS n
      ON n.nid = p.nid
      LEFT JOIN {uc_upsell_products} AS up
      ON up.nid = p.nid
      WHERE n.status = 1
      AND n.type IN ('". implode("','", uc_product_product_types()) ."')
      ORDER BY n.title";

  // Build a paged query for easy viewing
  $pagedquery = pager_query($query . tablesort_sql($header), $config['browser']['pager_size'], 0, NULL);

  while ($product = db_fetch_object($pagedquery)) {
    $related_products = unserialize($product->related_products);
    $titles = '';

    if (is_array($related_products)) {
      foreach ($related_products as $related) {
        if (!empty($titles)) $titles .= ', ';
        $title = db_result(db_query("SELECT title FROM {node} WHERE nid = %d", $related));
        $titles .= l($title, 'node/'. $related .'/edit', array('query' => 'destination='. $_GET['q'] .'?page='. $_GET['page'], 'fragment' => 'upsell'));
      }
    }

    $rows[] = array(
      array('data' => $product->title, 'class' => 'description-row'),
      array('data' => $titles, 'class' => 'description-row'),
      array('data' => l('Edit', 'node/'. $product->nid .'/edit', array('attributes' => array('class' => 'download-row'), 'query' => 'destination='. $_GET['q'] .'?page='. $_GET['page'], 'upsell'))),
    );
  }

  if (empty($rows)) {
    $rows[] = array(array('data' => t('No associations'), 'colspan' => 4));
  }

  // Show admin link if allowed...
  if (user_access('administer upsell settings')) {
    $output .= '<div class="help"><p>'. l(t('Edit upsell module settings'), 'admin/store/settings/upsell') .'</p></div>';
  }

  $output .= theme('table', $header, $rows) . theme('pager', NULL, $config['browser']['pager_size'], 0);

  return $output;
}


/**
 * Settings for the Upsell products
 */
function upsell_products_settings() {
  return drupal_get_form('uc_upsell_settings_form');
}


/**
 * Settings form
 */
function uc_upsell_settings_form() {

  $config = uc_upsell_get_config();

  $form['upsell_global'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => '<p>These settings affect both the "Upsell Items" block and Ubercart check-out pane. <b>Developers:</b> These settings also impact the behavior and output of the <i>get_associate_nids()</i> function.</p>
',
  );
  $form['upsell_global']['upsell_loose_msg_req'] = array(
    '#type' => 'checkbox',
    '#multiple' => FALSE,
    '#title' => t('Use default message when at least one upsell item is associated.'),
    '#default_value' => $config['global']['loose_msg_req'],
    '#description' => t('Check this option and the default message will be used, if at least one found product is associated with the target node (for product pages) or cart items (on the checkout page). <i>The "Upsell Items" block always uses the alternate message, when displayed with non-product pages.</i>'),
    '#required' => FALSE,
  );
  $form['upsell_global']['upsell_shuffle'] = array(
    '#type' => 'checkbox',
    '#multiple' => FALSE,
    '#title' => t('Shuffle related and supplemental nodes.'),
    '#default_value' => $config['global']['shuffle'],
    '#description' => t('Check this option and related nodes will be shuffled with any supplemental nodes, if any. <i>Note: Related and supplemental node groups are always randomized.</i>'),
    '#required' => FALSE,
  );
  /*$form['upsell_global']['upsell_buy_link'] = array(
    '#type' => 'checkbox',
    '#multiple' => FALSE,
    '#title' => t('Use a "Buy Now" cart link in the upsell blocks'),
    '#default_value' => $config['global']['buy_link'],
    '#description' => t('Unchecking this option will allow '),
    '#required' => FALSE,
  );*/

  if (module_exists('imagecache')) {
    $imagecache_presets = array();

    foreach (imagecache_presets() AS $preset) {
      $imagecache_presets[$preset['presetname']] = $preset['presetname'];
    }

    $form['upsell_global']['thumbnail_preset'] = array(
      '#type' => 'select',
      '#multiple' => FALSE,
      '#title' => t('Imagecache preset to use'),
      '#default_value' => uc_upsell_get_imagecache_preset(),
      '#options' => array_filter($imagecache_presets),
      '#description' => t('Choose the imagecache preset to apply to all blocks'),
      '#required' => FALSE,
    );
  }
  else {
    drupal_set_message(t('Install ImageCache module to get preset configurations.'));
  }

  $form['upsell_global']['types'] = array(
    '#type' => 'fieldset',
    '#title' => t('Node Types'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['upsell_global']['types']['upsell_types'] = array(
    '#type' => 'checkboxes',
    '#multiple' => TRUE,
    '#title' => t('Product types to allow associations with'),
    '#default_value' => $config['global']['types'],
    '#options' => uc_upsell_get_product_names(uc_product_node_info()),
    '#description' => t('Choose which product types can be selected in the Node Edit form.'),
    '#required' => FALSE,
  );
  $form['upsell_global']['upsell_taxonomy'] = array(
    '#type' => 'select',
    '#title' => t('Handle Related Products based on Taxonomy'),
    '#default_value' => $config['global']['taxonomy'] ? $config['global']['taxonomy'] : 'ignore',
    '#description' => t('How should we deal with Products that are related by taxonomy terms? Use this feature for smarter Upselling.
      <br />- <strong>Ignore: </strong> <em>Default</em>. Displays products specified in the Node Edit form, plus randoms.
      <br />- <strong>Override: </strong> Displays products related by taxonomy first, ignoring products specified in Node Edit Form.
      <br />- <strong>Add: </strong> Displays products specified by Node Edit Form, plus nodes related by taxonomy, then randoms if needed.'),
    '#options' => array(
      'ignore' => t('Do nothing'),
      'override' => t('Override them'),
      'add' => t('Add them'),
    ),
  );
  $form['upsell_pane'] = array(
    '#type' => 'fieldset',
    '#title' => t('Cart Pane'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => '<p>The "Upsell Items" pane displays products related to items in a user\'s cart. You can edit the order and presence of cart panes via the '. l("cart settings pane", 'admin/store/settings/cart/edit/panes') .'.</p>
<p></p>',
  );
  $form['upsell_pane']['upsell_msg_related'] = array(
    '#type' => 'textfield',
    '#title' => 'Default checkout message',
    '#description' => 'Message displayed when all items are associated to the user\'s cart.',
    '#default_value' => $config['pane']['msg_related'],
    '#length' => 40,
    '#required' => FALSE,
  );
  $form['upsell_pane']['upsell_msg_mixed'] = array(
    '#type' => 'textfield',
    '#title' => 'Alternate checkout message',
    '#description' => 'Message displayed when one or more items have no association to the user\'s cart.',
    '#default_value' => $config['pane']['msg_mixed'],
    '#length' => 40,
    '#required' => FALSE,
  );
  $form['upsell_pane']['upsell_max'] = array(
    '#type' => 'textfield',
    '#title' => 'Number of products',
    '#description' => 'How many products to suggest at checkout',
    '#default_value' => $config['pane']['max'],
    '#length' => 2,
    '#size' => 3,
    '#required' => TRUE,
  );
  $form['upsell_pane']['upsell_use_random'] = array(
    '#type' => 'checkbox',
    '#multiple' => FALSE,
    '#title' => t('Supplement with random products'),
    '#default_value' => $config['pane']['use_random'],
    '#description' => t('Check this option and random products will be listed when there are not enough associated items. <strong>NOTE</strong> This will override your Upsell Product Type settings, and will use any Ubercart product types available!'),
    '#required' => FALSE,
  );
  $form['upsell_browser'] = array(
    '#type' => 'fieldset',
    '#title' => t('Associations Browser'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => '<p>Configure the behavior and appearance of the '. l('product associations browser', 'admin/store/products/upsell') .'.</p>',
  );
  $form['upsell_browser']['upsell_pager_size'] = array(
    '#type' => 'select',
    '#title' => t('Number of products to list per page'),
    '#description' => t('Change this to suit your paging needs'),
    '#default_value' => $config['browser']['pager_size'],
    '#options' => drupal_map_assoc(uc_range(10, 100, 10)),
    '#required' => TRUE,
  );
  $form['submit'] = array('#type' => 'submit',
    '#value' => 'Save Settings',
    '#weight' => 10,
  );
  $form['upsell_block_link'] = array(
    '#value' => '<div><p>You may also '. l('configure the "Upsell Items" block', 'admin/build/block/configure/uc_upsell/') .'.</p></div>',
    '#weight' => 11
  );

  return $form;
}

/**
 * Submit the settings form
 */
function uc_upsell_settings_form_submit($form, &$form_state) {
  $config = uc_upsell_get_config();
  $config['global']['loose_msg_req'] = $form_state['values']['upsell_loose_msg_req'];
  $config['global']['shuffle'] = $form_state['values']['upsell_shuffle'];
  $config['global']['types'] = $form_state['values']['upsell_types'];
  $config['global']['thumbnail_preset'] = array($form_state['values']['thumbnail_preset']);
  $config['global']['taxonomy'] = $form_state['values']['upsell_taxonomy'];
  $config['pane']['msg_related'] = $form_state['values']['upsell_msg_related'];
  $config['pane']['msg_mixed'] = $form_state['values']['upsell_msg_mixed'];
  $config['pane']['max'] = $form_state['values']['upsell_max'];
  $config['pane']['use_random'] = $form_state['values']['upsell_use_random'];
  $config['browser']['pager_size'] = $form_state['values']['upsell_pager_size'];
  uc_upsell_set_config($config);
  drupal_set_message(t('Upsell configuration was updated.'));
}


/**
 * Get variables
 */
function uc_upsell_get_config() {
  $config = variable_get('uc_upsell_config', uc_upsell_get_config_defaults());
  return $config;
}

/**
 * Return the config defaults
 */
function uc_upsell_get_config_defaults() {
  $uc_upsell_config_defaults = array(
    'global' => array(
      'loose_msg_req' => FALSE,
      'shuffle' => TRUE,
      'types' => array('product' => 'Product'),
      'thumbnail_preset' => array('uc_thumbnail'),
      'taxonomy' => FALSE,
    ),
    'pane' => array(
      'max' => 5,
      'use_random' => TRUE,
      'msg_related' => 'Users who bought items in your cart also purchased:',
      'msg_mixed' => 'Users who bought items in your cart also purchased:',
    ),
    'block' => array(
      'max' => 5,
      'enable' => 0,
      'product' => array(
        'use_cart' => TRUE,
        'use_random' => TRUE,
        'msg_related' => 'Related products',
        'msg_mixed' => 'Also available',
      ),
      'node' => array(
        'req_cart' => FALSE,
        'source' => 1,
        'msg_related' => 'Compliment your order with...',
        'msg_mixed' => 'Must see products',
      ),
    ),
    'browser' => array(
      'pager_size' => 10,
      'truncate' => TRUE,
    ),
  );
  return $uc_upsell_config_defaults;
}


/**
 * Save uc_upsell config to drupal variable
 */
function uc_upsell_set_config($config) {
  variable_set('uc_upsell_config', $config);
}


/**
 * Revert parts of upsell variable's root keys
 */
function uc_upsell_reset_config($delkeys = array()) {

  if (!is_array($delkeys)) {
    $delkeys = array($delkeys);
  }

  $config = uc_upsell_get_config();
  $original = uc_upsell_get_config_defaults();

  // If no specific keys to delete...
  if (empty($delkeys)) {
    $config = $original;
  }
  else { // Otherwise, only replace targeted keys...
    foreach ($delkeys as $key) {
      if (array_key_exists($key, $original)) {
        $config[$key] = $original[$key];
      }
    }
  }
  uc_upsell_set_config($config);
  drupal_set_message('Upsell configuration was reset to defaults.');
}
